﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using MvcApplication4.Interfaces;

namespace MvcApplication4
{

    public class MessagesDataSource
    {
        private static readonly CloudStorageAccount StorageAccount;
        private readonly MyContext _context;

        static MessagesDataSource()
        {
            StorageAccount = CloudStorageAccount.Parse("DefaultEndpointsProtocol=http;AccountName=10st0001;AccountKey=SR+2lqHxqSfYGUR4DlHJXnuNlZZARoqTcFWHyMGXlMWlGWB7ZYTJO/fwGzPBtbr0RixQ0I0f7px7tKipbMAPsg==");

            //CloudTableClient.CreateTablesFromModel(
            //    typeof(MyContext),
            //    StorageAccount.TableEndpoint.AbsoluteUri,
            //    StorageAccount.Credentials);

            CloudTableClient cloudTableClient = StorageAccount.CreateCloudTableClient();
            cloudTableClient.CreateTableIfNotExist("Messages");

        }

        public MessagesDataSource()
        {
            _context = new MyContext(StorageAccount.TableEndpoint.AbsoluteUri, StorageAccount.Credentials);
            _context.RetryPolicy = RetryPolicies.Retry(3, TimeSpan.FromSeconds(1));
        }

        public IEnumerable<MyMessageEntry> Select()
        {
            //var results = from g in this._context.MyEntity 
            //              where g.PartitionKey == DateTime.UtcNow.ToString("MMddyyyy")
            //              select g;
            // return results;

            return _context.MyEntity;
        }

        public void AddMessage(MyMessageEntry newItem)
        {
            _context.AddObject("Messages", newItem);
            _context.SaveChanges();
        }
    }

    public class MyContext : TableServiceContext
    {
        public MyContext(string baseAddress, StorageCredentials credentials)
            : base(baseAddress, credentials)
        {
        }

        public IQueryable<MyMessageEntry> MyEntity
        {
            get
            {
                return this.CreateQuery<MyMessageEntry>("Messages");
            }
        }
    }

    public sealed class MyMessageEntry : TableServiceEntity
    {
        public MyMessageEntry()
        {
            /*
             * Tables are partitioned to support load balancing across storage nodes. A table's entities are organized by partition. 
             * A partition is a consecutive range of entities possessing the same partition key value. The partition key is a unique identifier 
             * for the partition within a given table, specified by the PartitionKey property. 
             * The partition key forms the first part of an entity's primary key. The partition key may be a string value up to 1 KB in size.
             */
            PartitionKey = DateTime.UtcNow.ToString("MMddyyyy");

            /*
             * The second part of the primary key is the row key, specified by the RowKey property. The row key is a unique identifier
             * for an entity within a given partition. Together the PartitionKey and RowKey uniquely identify every entity within a table.
             * */
            RowKey = string.Format("{0:10}_{1}", DateTime.MaxValue.Ticks - DateTime.Now.Ticks, Guid.NewGuid());
        }

        public string Message { get; set; }
    }


    public class SimpleStorageTable:ISimpleStorage
    {
        readonly MessagesDataSource _messagesDataSource;

        public SimpleStorageTable ()
        {
            _messagesDataSource = new MessagesDataSource();
        }

        public List<string> Messages
        {
            get
            {
                
            }
        }

        public void AddMessage(string message)
        {
           
        }
    }
}